package com.hungraim.ltc.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ltc.api.AttachFeignService;
import com.hungraim.ltc.dao.*;
import com.hungraim.ltc.pojo.entity.SysConfig.ChFndAttach;
import com.hungraim.ltc.pojo.entity.disable.ChCfgSrvEmpType;
import com.hungraim.ltc.pojo.entity.disable.ChSrvOrgan;
import com.hungraim.ltc.pojo.entity.disable.ChSrvOrganApply;
import com.hungraim.ltc.pojo.entity.riskLevel.ChSrvempRiskLevel;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvOrganEmp;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvOrganEmpApply;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvTaskDetail;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvTaskDetailDelete;
import com.hungraim.ltc.pojo.vo.disable.*;
import com.hungraim.ltc.pojo.vo.system.AttachUpdateVo;
import com.hungraim.ltc.service.OrganService;
import com.hungraim.ltc.service.SrvOrganEmpService;
import com.hungraim.ltc.util.AttachType;
import com.hungraim.ltc.util.Result;
import com.hungraim.ltc.util.ResultCode;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
@AllArgsConstructor
@Slf4j
public class SrvOrganEmpServiceImpl implements SrvOrganEmpService {

    private final ChSrvOrganEmpMapper chSrvOrganEmpMapper;

    private final ChSrvOrganEmpApplyMapper chSrvOrganEmpApplyMapper;


    private final OrganService organService;

    private final AttachFeignService attachFeignService;

    private final SrvEmpTypeMapper srvEmpTypeMapper;

    private final SrvOrganApplyMapper srvOrganApplyMapper;

    @Autowired
    private ChSrvempRiskLevelMapper chSrvempRiskLevelMapper;

    private final ChSrvTaskDetailMapper chSrvTaskDetailMapper;

    private final ChSrvTaskDetailDeleteMapper chSrvTaskDetailDeleteMapper;



    @Override
    public Page<SrvOrganEmpListVo> getSrvOrganEmpApplyListVo(Page<SrvOrganEmpListVo> page, Long organId, Boolean isInvolveChild, String empName, String srvOrganName, String certifNum, Short isLock, Short empType,Long srvOrganId) {
        List<Long> organIds = organService.getOrganIds(organId, isInvolveChild);
        return chSrvOrganEmpApplyMapper.getSrvOrganApplyEmpListVo(page, organIds, empName, srvOrganName, certifNum, isLock, empType,srvOrganId);
    }

    @Override
    public void updateSrvEmpStatus(Long empId, Short isLock) {
        ChSrvOrganEmp chSrvOrganEmp = new ChSrvOrganEmp();
        chSrvOrganEmp.setEmpId(empId);
        chSrvOrganEmp.setIsLock(isLock);
        chSrvOrganEmpMapper.updateById(chSrvOrganEmp);
        ChSrvOrganEmpApply chSrvOrganEmpApply  = new ChSrvOrganEmpApply();
        chSrvOrganEmpApply.setEmpId(empId);
        chSrvOrganEmpApply.setIsLock(isLock);
        chSrvOrganEmpApplyMapper.updateById(chSrvOrganEmpApply);
    }

    @Override
    public SrvOrganEmpVo getSrvOrganEmp(Long empId) {
        SrvOrganEmpVo srvOrganEmpVo = new SrvOrganEmpVo();
        ChSrvOrganEmpApply chSrvOrganEmpApply = chSrvOrganEmpApplyMapper.selectById(empId);
        Long empTypeId = chSrvOrganEmpApply.getEmpTypeId();
        if(empTypeId!=null){
            ChCfgSrvEmpType chCfgSrvEmpType = srvEmpTypeMapper.selectById(empTypeId);
            srvOrganEmpVo.setEmpCateId(chCfgSrvEmpType.getEmpCateId());
        }else{
            srvOrganEmpVo.setEmpCateId(0);
        }
        Long srvOrganId = chSrvOrganEmpApply.getSrvOrganId();
        ChSrvOrganApply chSrvOrganApply = srvOrganApplyMapper.selectById(srvOrganId);
        BeanUtils.copyProperties(chSrvOrganEmpApply, srvOrganEmpVo);
        srvOrganEmpVo.setUnifiedSocialCreditCode(chSrvOrganApply.getUnifiedSocialCreditCode());
        try {
            Result<List<ChFndAttach>> listResult = attachFeignService.selectAttachs(empId, AttachType.SRV_ORGAN_EMP.getUploadType());
            if (ResultCode.SUCCESS.getCode().equals(listResult.getCode())) {
                srvOrganEmpVo.setFileDataList(listResult.getData());
            } else {
                log.error("请求获取护理人员上传文件出错，出错信息：{}", listResult.getMsg());
            }
        } catch (Exception e) {
            log.error("请求获取护理人员上传文件出错，出错信息：{}", e.getMessage());
        }
        return srvOrganEmpVo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveOrUpdateSrvOrganEmp(SrvOrganEmpVo srvOrganEmpVo) {
        Long empId = srvOrganEmpVo.getEmpId();
        ChSrvOrganEmpApply chSrvOrganEmpApply = new ChSrvOrganEmpApply();
        BeanUtils.copyProperties(srvOrganEmpVo, chSrvOrganEmpApply);
        if (null == empId) {
            // 新增
            chSrvOrganEmpApply.setEffTime(new Date());
            chSrvOrganEmpApply.setIsLock((short) 0);
            chSrvOrganEmpApplyMapper.insert(chSrvOrganEmpApply);
            updateAttachs(srvOrganEmpVo, chSrvOrganEmpApply.getEmpId());

            ChSrvempRiskLevel chSrvempRiskLevel = new ChSrvempRiskLevel();
            chSrvempRiskLevel.setRiskLevel(0L);
            chSrvempRiskLevel.setSrvEmpId(Long.valueOf(chSrvOrganEmpApply.getEmpId()));
            chSrvempRiskLevel.setStatus(0);
            chSrvempRiskLevel.setFcd(new Date());
            chSrvempRiskLevelMapper.insert(chSrvempRiskLevel);

        } else {
            // 修改
            chSrvOrganEmpApplyMapper.updateById(chSrvOrganEmpApply);
            updateAttachs(srvOrganEmpVo, empId);
        }
    }

    // 抽取方法
    private void updateAttachs(SrvOrganEmpVo srvOrganEmpVo, Long empId) {
        AttachUpdateVo attachUpdateVo = new AttachUpdateVo();
        attachUpdateVo.setChFndAttachs(srvOrganEmpVo.getFileDataList());
        attachUpdateVo.setCtrlId(empId);
        attachUpdateVo.setAttachType(AttachType.SRV_ORGAN_EMP.getUploadType());
        attachFeignService.updateAttachs(attachUpdateVo);
    }


    @Override
    public Workbook srvOrganExport(Long organId,Long srvOrganId, Boolean isInvolveChild, String empName, String srvOrganName, String certifNum,
                                   Short isLock, Short empType) {
        List<Long> organIds = organService.getOrganIds(organId, isInvolveChild);
        List<SrvOrganEmpExportVo> srvOrganEmpExportList = chSrvOrganEmpApplyMapper.getSrvOrganEmpApplyExportList(organIds,srvOrganId, empName, srvOrganName, certifNum, isLock, empType);
        ExportParams exportParams = new ExportParams();
        exportParams.setType(ExcelType.XSSF); // 对应xlsx
        Workbook workbook = ExcelExportUtil.exportExcel(exportParams, SrvOrganEmpExportVo.class, srvOrganEmpExportList);

        return workbook;
    }

    @Override
    public Page<ChSrvTaskDetail2Vo> chSrvTaskDetailQuery(Page<ChSrvTaskDetail2Vo> page, Long empId) {
        Page<ChSrvTaskDetail2Vo> chSrvTaskDetailVoPage = chSrvTaskDetailMapper.chSrvTaskDetailQuery(page, empId);
        chSrvTaskDetailVoPage.getRecords().stream().filter(chSrvTaskDetail -> {
            if (null != chSrvTaskDetail.getSevStarTime()){
                chSrvTaskDetail.setSevStarTime(chSrvTaskDetail.getSevStarTime().substring(0,19));
            }
            if (null != chSrvTaskDetail.getSevEndTime()){
                chSrvTaskDetail.setSevEndTime(chSrvTaskDetail.getSevEndTime().substring(0,19));
            }
            return true;
        }).collect(Collectors.toList());
        return chSrvTaskDetailVoPage;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int chSrvTaskDetailDelete(long taskDetailId,String userName) {
        ChSrvTaskDetail chSrvTaskDetail = chSrvTaskDetailMapper.selectById(taskDetailId);
        ChSrvTaskDetailDelete chSrvTaskDetailDelete = new ChSrvTaskDetailDelete();
        BeanUtils.copyProperties(chSrvTaskDetail,chSrvTaskDetailDelete);
        chSrvTaskDetailDelete.setDeleteDate(new Date());
        chSrvTaskDetailDelete.setDeleteName(userName);
        chSrvTaskDetailDeleteMapper.insert(chSrvTaskDetailDelete);
        return chSrvTaskDetailMapper.deleteById(taskDetailId);
    }

    @Override
    public void auditSrvOrganEmp(Integer userId, Long empId, Integer auditStatus) {
        ChSrvOrganEmpApply chSrvOrganEmpApply= new ChSrvOrganEmpApply();
        chSrvOrganEmpApply.setEmpId(empId);
        chSrvOrganEmpApply.setAuditStatus(auditStatus.shortValue());
        chSrvOrganEmpApply.setAuditDate(new Date());
        chSrvOrganEmpApply.setAuditUserId(userId);
        int result = chSrvOrganEmpApplyMapper.updateById(chSrvOrganEmpApply);
        //审核通过 同步到主表
        if (result>0) {
            //查出申请表的信息同步到主表
            chSrvOrganEmpApply = chSrvOrganEmpApplyMapper.selectOne(new LambdaQueryWrapper<ChSrvOrganEmpApply>()
                    .eq(ChSrvOrganEmpApply::getEmpId,empId));
            //主表
            ChSrvOrganEmp querySrvOrganEmp = chSrvOrganEmpMapper.selectOne(new LambdaQueryWrapper<ChSrvOrganEmp>()
                    .eq(ChSrvOrganEmp::getEmpId,empId));
            ChSrvOrganEmp chSrvOrganEmp = new ChSrvOrganEmp();
            BeanUtils.copyProperties(chSrvOrganEmpApply,chSrvOrganEmp);
            if(querySrvOrganEmp==null){
                chSrvOrganEmpMapper.insert(chSrvOrganEmp);
            }else{
                chSrvOrganEmpMapper.updateById(chSrvOrganEmp);
            }
        }
    }

}
